@@ -91,3 +91,13 @@ class DateTimeDimension(Dimension):
         if isinstance(value, date):
             return datetime(value.year, value.month, value.day)
         return datetime.fromisoformat(value)
+
+
+class ArrayDimension(Dimension):
+
+    _type = list
+    default = []
+
+    def __init__(self, name: Optional[str] = None, null: bool = False, dimension: Dimension = None):
+        super().__init__(name, null)
+        self.dimension = dimension
