@@ -17,6 +17,7 @@ from hcube.api.models.aggregation import (
 )
 from hcube.api.models.cube import Cube
 from hcube.api.models.dimensions import (
+    ArrayDimension,
     DateDimension,
     DateTimeDimension,
     Dimension,
@@ -30,6 +31,10 @@ from hcube.api.models.filters import (
     IsNullFilter,
     ListFilter,
     NegativeListFilter,
+    Or,
+    OverlapFilter,
+    SubstringFilter,
+    SubstringMultiValueFilter,
 )
 from hcube.api.models.materialized_views import AggregatingMaterializedView
 from hcube.api.models.metrics import FloatMetric, IntMetric, Metric
@@ -52,6 +57,7 @@ class TableMetaParams:
     sign_col: str = "sign"
     primary_key: [str] = field(default_factory=list)
     sorting_key: [str] = field(default_factory=list)
+    partition_key: [str] = field(default_factory=list)
     indexes: [IndexDefinition] = field(default_factory=list)
     dictionaries: [DictionaryDefinition] = field(default_factory=list)
     # Using a skip index with FINAL queries may lead to incorrect results in some cases
@@ -386,6 +392,16 @@ class ClickhouseCubeBackend(CubeBackend):
         a dictionary with the parameters that should be passed to the query during execution
         for proper escaping.
         """
+        # combinators first
+        if isinstance(fltr, Or):
+            queries = []
+            params = {}
+            for subfilter in fltr.filters:
+                query, subparams = self._ch_filter(subfilter)
+                queries.append(query)
+                params.update(subparams)
+            return f'({" OR ".join(queries)})', params
+        # then plain filters
         key = f"_where_{id(fltr)}_{fltr.dimension.name}"
         if isinstance(fltr, ListFilter):
             return f"{fltr.dimension.name} IN (%({key})s)", {key: fltr.values}
@@ -398,6 +414,16 @@ class ClickhouseCubeBackend(CubeBackend):
             return f"{fltr.dimension.name} {fltr.comparison.value} %({key})s", {key: fltr.value}
         if isinstance(fltr, EqualityFilter):
             return f"{fltr.dimension.name} = %({key})s", {key: fltr.value}
+        if isinstance(fltr, SubstringFilter):
+            fn = "positionUTF8" if fltr.case_sensitive else "positionCaseInsensitiveUTF8"
+            return f"{fn}({fltr.dimension.name}, %({key})s) > 0", {key: fltr.value}
+        if isinstance(fltr, SubstringMultiValueFilter):
+            fn = (
+                "multiSearchAnyUTF8" if fltr.case_sensitive else "multiSearchAnyCaseInsensitiveUTF8"
+            )
+            return f"{fn}({fltr.dimension.name}, %({key})s)", {key: fltr.values}
+        if isinstance(fltr, OverlapFilter):
+            return f"hasAny({fltr.dimension.name}, %({key})s)", {key: fltr.values}
         raise ValueError(f"unsupported filter {fltr.__class__}")
 
     @classmethod
@@ -464,20 +490,27 @@ class ClickhouseCubeBackend(CubeBackend):
         if engine == "CollapsingMergeTree":
             engine = f"{engine}({meta.sign_col})"
             field_part += f", {meta.sign_col} Int8 default 1"
+        partition_part = ""
+        if meta.partition_key:
+            partition_part = f"PARTITION BY ({','.join(meta.partition_key)})"
         allow_nullable_key = any(dim.null for dim in cube._dimensions.values())
         settings_part = "SETTINGS allow_nullable_key = 1" if allow_nullable_key else ""
         command = (
             f"CREATE TABLE IF NOT EXISTS {self.database}.{name} ({field_part}) "
             f"ENGINE = {engine} "
             f"PRIMARY KEY ({primary_key}) "
-            f"ORDER BY ({sorting_key})"
+            f"ORDER BY ({sorting_key}) "
+            f"{partition_part} "
             f"{settings_part};"
         )
         logger.debug(command)
         with self.pool.get_client() as client:
             client.execute(command)
 
-    def _ch_type(self, dimension: Union[Dimension, Metric]):
+    def _ch_type(self, dimension: Union[Dimension, Metric]) -> str:
+        if isinstance(dimension, ArrayDimension):
+            subtype = self._ch_type(dimension.dimension)
+            return f"Array({subtype})"
         for dim_cls, ch_type in self.dimension_type_map:
             if isinstance(dimension, (IntDimension, IntMetric)):
                 sign = "U" if not dimension.signed else ""
@@ -558,10 +591,10 @@ class ClickhouseCubeBackend(CubeBackend):
                         logger.info(
                             'Dictionary "%s" definition has changed, recreating', dict_def.name
                         )
-                        client.execute(dict_def.drop_sql())
+                        client.execute(dict_def.drop_sql(database=self.database))
                     else:
                         continue
-                client.execute(dict_def.definition_sql())
+                client.execute(dict_def.definition_sql(database=self.database))
 
     def _drop_dictionaries(self, cube: Type[Cube]):
         meta = self.get_table_meta(cube)
@@ -584,13 +617,15 @@ class ClickhouseCubeBackend(CubeBackend):
             return transform.name, select
         if isinstance(transform, StoredMappingTransform):
             select = (
-                f"dictGetOrDefault('{transform.mapping_name}', '{transform.mapping_field}', "
-                f"toUInt64({transform.dimension.name}), {transform.dimension.name}) "
-                f"AS {transform.name}"
+                f"dictGet('{self.database}.{transform.mapping_name}', "
+                f"'{transform.mapping_field}', "
+                f"toUInt64({transform.dimension.name})) AS {transform.name}"
             )
             return transform.name, select
         raise ValueError(f"Unsupported transform {transform.__class__} in the clickhouse backend")
 
     def _drop_table(self, cube: Type[Cube]):
         with self.pool.get_client() as client:
-            client.execute(f"DROP TABLE IF EXISTS {self.database}.{self.cube_to_table_name(cube)}")
+            client.execute(
+                f"DROP TABLE IF EXISTS {self.database}.{self.cube_to_table_name(cube)} SYNC"
+            )
