@@ -10,6 +10,7 @@ from hcube.api.exceptions import ConfigurationError
 from hcube.api.models.aggregation import AggregationOp, ArrayAgg, Count
 from hcube.api.models.cube import Cube
 from hcube.api.models.dimensions import (
+    ArrayDimension,
     DateDimension,
     DateTimeDimension,
     Dimension,
@@ -23,6 +24,10 @@ from hcube.api.models.filters import (
     IsNullFilter,
     ListFilter,
     NegativeListFilter,
+    Or,
+    OverlapFilter,
+    SubstringFilter,
+    SubstringMultiValueFilter,
 )
 from hcube.api.models.metrics import FloatMetric, IntMetric, Metric
 from hcube.api.models.query import CubeQuery
@@ -218,6 +223,16 @@ class PostgresCubeBackend(CubeBackend):
         a dictionary with the parameters that should be passed to the query during execution
         for proper escaping.
         """
+        # combinators first
+        if isinstance(fltr, Or):
+            queries = []
+            params = {}
+            for subfilter in fltr.filters:
+                query, subparams = self._ch_filter(subfilter)
+                queries.append(query)
+                params.update(subparams)
+            return " OR ".join(queries), params
+        # then plain filters
         # add id(fltr) to the name because there may be more than one filter for the same dimension
         key = f"_where_{id(fltr)}_{fltr.dimension.name}"
         if isinstance(fltr, ListFilter):
@@ -231,6 +246,16 @@ class PostgresCubeBackend(CubeBackend):
             return f"{fltr.dimension.name} {fltr.comparison.value} %({key})s", {key: fltr.value}
         if isinstance(fltr, EqualityFilter):
             return f"{fltr.dimension.name} = %({key})s", {key: fltr.value}
+        if isinstance(fltr, SubstringFilter):
+            op = "LIKE" if fltr.case_sensitive else "ILIKE"
+            return f"{fltr.dimension.name} {op} %({key})s", {key: f"%{fltr.value}%"}
+        if isinstance(fltr, SubstringMultiValueFilter):
+            op = "LIKE" if fltr.case_sensitive else "ILIKE"
+            return " OR ".join(
+                f"{fltr.dimension.name} {op} %({key}_{i})s" for i in range(len(fltr.values))
+            ), {f"{key}_{i}": f"%{value}%" for i, value in enumerate(fltr.values)}
+        if isinstance(fltr, OverlapFilter):
+            return f"{fltr.dimension.name} && %({key})s", {key: fltr.values}
         raise ValueError(f"unsupported filter {fltr.__class__}")
 
     def _agg_name(self, agg: AggregationOp):
@@ -258,10 +283,18 @@ class PostgresCubeBackend(CubeBackend):
         logger.debug(command)
         self._execute(command)
 
-    def _ch_type(self, dimension: Union[Dimension, Metric]):
+    def _ch_type(self, dimension: Union[Dimension, Metric]) -> str:
+        a, b = self._ch_type_parts(dimension)
+        return f"{a} {b}" if b else a
+
+    def _ch_type_parts(self, dimension: Union[Dimension, Metric]) -> (str, str):
+        if isinstance(dimension, ArrayDimension):
+            subtype, _subnull = self._ch_type_parts(dimension.dimension)
+            # we ignore the null definition
+            return f"{subtype}[]", ""
         for dim_cls, ch_type in self.dimension_type_map:
             if isinstance(dimension, dim_cls):
                 if hasattr(dimension, "null") and dimension.null:
-                    return ch_type
-                return f"{ch_type} NOT NULL"
+                    return ch_type, ""
+                return ch_type, "NOT NULL"
         raise ValueError("unsupported dimension: %s", dimension.__class__)
